/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * <p>
 * Defines an abstract class which implements the <code>NodeDiscovery</code> interface and is useful for
 * <code>NodeDiscovery</code> implementations that are also a <code>Tag</code>.
 * </p>
 *
 * <p>
 * This abstract class simplifies the work required when an implementation of the <code>NodeDiscovery</code> will also
 * provide services as a tag and provides an attribute to set the title. This abstract class will get the current node
 * and then set the result as a page scope attribute using <code>BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE</code> for
 * further usage.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public abstract class AbstractNodeDiscovery extends TagSupport implements NodeDiscovery {
    /**
     * <p>
     * Represents the title associated with the node.
     * </p>
     *
     * <p>
     * This variable is set by the constructor initially to <code>null</code> and is mutable. This value can be
     * <code>null</code> if the title should default to the title found in the <code>PathDiscovery</code>
     * implementation. This value should not be <code>null</code> if it's a node that is part of a path returned by
     * the <code>PathDiscovery</code> implementation. This value should never be an empty string.
     * </p>
     */
    private String title = null;

    /**
     * <p>
     * Creates a new instance of <code>AbstractNodeDiscovery</code> class. Nothing will be done in this constructor.
     * </p>
     */
    protected AbstractNodeDiscovery() {
        // do nothing here
    }

    /**
     * <p>
     * Gets the current node and saves it as an attribute to the current page context using page scope.
     * </p>
     *
     * @return Always SKIP_BODY.
     *
     * @throws BreadCrumbException if it fails to get the current node.
     */
    public int doStartTag() throws BreadCrumbException {
        BreadCrumbNode currentNode = getCurrentNode(this.pageContext);

        if (currentNode != null) {
            this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, currentNode,
                PageContext.PAGE_SCOPE);
        }

        return SKIP_BODY;
    }

    /**
     * <p>
     * Gets the title of page represented by the node.
     * </p>
     *
     * @return A possibly <code>null</code>, non-empty (trim'd) string representing the title of page represented by
     *         the node.
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>
     * Sets the title of page represented by the node.
     * </p>
     *
     * @param title the title of page represented by the node. If the title is an empty string, it will be regarded as
     *        <code>null</code>.
     */
    public void setTitle(String title) {
        this.title = Helper.convertEmptyString(title);
    }
}
