/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import com.topcoder.util.config.ConfigManager;

import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.NodeMatcher;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import junit.framework.Assert;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

import java.lang.reflect.Field;

import java.util.Iterator;


/**
 * <p>
 * Defines helper methods used in tests.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public final class UnitTestHelper {
    /**
     * <p>
     * Creates a new instance of <code>UnitTestHelper</code> class. The private constructor prevents the creation of a
     * new instance.
     * </p>
     */
    private UnitTestHelper() {
    }

    /**
     * <p>
     * Gets the value of a private field in the given class. The field has the given name. The value is retrieved from
     * the given instance.
     * </p>
     *
     * @param type the class which the private field belongs to.
     * @param instance the instance which the private field belongs to.
     * @param name the name of the private field to be retrieved.
     *
     * @return the value of the private field or <code>null</code> if any error occurs.
     */
    public static Object getPrivateField(Class type, Object instance, String name) {
        Field field = null;
        Object obj = null;

        try {
            // Get the reflection of the field and get the value
            field = type.getDeclaredField(name);
            field.setAccessible(true);
            obj = field.get(instance);
        } catch (NoSuchFieldException e) {
            // ignore
        } catch (IllegalAccessException e) {
            // ignore
        } finally {
            if (field != null) {
                // Reset the accessibility
                field.setAccessible(false);
            }
        }

        return obj;
    }

    /**
     * <p>
     * Sets the value of a private field in the given class. The field has the given name. The value is to set into the
     * the given instance.
     * </p>
     *
     * @param type the class which the private field belongs to.
     * @param instance the instance which the private field belongs to.
     * @param name the name of the private field to set.
     * @param object the object to set the private field.
     */
    public static void setPrivateField(Class type, Object instance, String name, Object object) {
        Field field = null;

        try {
            // Get the reflection of the field and set the value
            field = type.getDeclaredField(name);
            field.setAccessible(true);
            field.set(instance, object);
        } catch (NoSuchFieldException e) {
            // ignore
        } catch (IllegalAccessException e) {
            // ignore
        } finally {
            if (field != null) {
                // Reset the accessibility
                field.setAccessible(false);
            }
        }
    }

    /**
     * <p>
     * clear the config.
     * </p>
     *
     * @throws Exception unexpected exception.
     */
    public static void clearConfig() throws Exception {
        ConfigManager configManager = ConfigManager.getInstance();

        for (Iterator iter = configManager.getAllNamespaces(); iter.hasNext();) {
            configManager.removeNamespace((String) iter.next());
        }
    }

    /**
     * <p>
     * add the default config file.
     * </p>
     *
     * @throws Exception unexpected exception.
     */
    public static void addConfig() throws Exception {
        addConfig("BreadCrumbTrailTagConfig.xml");
    }

    /**
     * <p>
     * add the config of given config file.
     * </p>
     *
     * @param configFile the given config file.
     *
     * @throws Exception unexpected exception.
     */
    public static void addConfig(String configFile) throws Exception {
        UnitTestHelper.clearConfig();

        ConfigManager configManager = ConfigManager.getInstance();
        configManager.add(configFile);
    }

    /**
     * Retrieve the content from the existing file.
     *
     * @param file the file to retrieve the content.
     *
     * @return the content of the file. One line should be one element of the list.
     *
     * @throws Exception unexpected exception.
     */
    public static String getFileContent(File file) throws Exception {
        BufferedReader reader = new BufferedReader(new FileReader(file));
        String line = null;
        StringBuffer content = new StringBuffer();

        while ((line = reader.readLine()) != null) {
            content.append(line);
        }

        reader.close();

        return content.toString();
    }

    /**
     * Builds the nodeMatcher for testing.
     *
     * @param matcherName the name of the node matcher.
     *
     * @return the created node matcher.
     *
     * @throws Exception any exception to JUnit.
     */
    public static NodeMatcher buildNodeMatcher(String matcherName) throws Exception {
        return new RegexNodeMatcher("Title" + matcherName, "Url" + matcherName, "Url" + matcherName);
    }

    /**
     * Builds the bread crumb node for testing.
     *
     * @param nodeName the name of the bread crumb node.
     *
     * @return the created bread crumb node.
     */
    public static BreadCrumbNode buildBreadCrumbNode(String nodeName) {
        return new BreadCrumbNode("Url" + nodeName, "Title" + nodeName);
    }

    /**
     * Asserts two bread crumb nodes are equal.
     *
     * @param message the error message when the two nodes are different.
     * @param expected the expected node.
     * @param actual the actual node.
     */
    public static void assertNodeEquals(String message, BreadCrumbNode[] expected, BreadCrumbNode[] actual) {
        Assert.assertEquals(message, expected.length, actual.length);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(message, expected[i].getTitle(), actual[i].getTitle());
            Assert.assertEquals(message, expected[i].getUrl(), actual[i].getUrl());
        }
    }
}
